#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# === This file is part of Calamares - <http://github.com/calamares> ===
#
#   Copyright 2014, Anke Boersma <demm@kaosx.us>
#
#   Calamares is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Calamares is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with Calamares. If not, see <http://www.gnu.org/licenses/>.

import os
import shutil

import libcalamares


def run():
    """ Create Distribution specific settings for users """
    install_path = libcalamares.globalstorage.value("rootMountPoint")
    user = libcalamares.globalstorage.value("username")

    print('create common dirs')
    common_dirs = [
        'Desktop',
        '.kde4/autostart',
        '.kde4/env',
        '.kde4/share/config',
        '.kde4/share/apps/konqueror',
        '.kde4/share/apps/homerun',
        '.local/share/applications',
        '.kde4/share/kde4/services/searchproviders',
        '.config/autostart'
    ]
    for d in common_dirs:
        libcalamares.utils.chroot_call(
            ['/usr/bin/mkdir', '-p', '/home/%s/%s' % (user,  d)])

    print('setup distribution specific settings')
    distro_settings = [
        ('ksplashrc', '.kde4/share/config/'),
        ('kcminputrc', '.kde4/share/config/'),
        ('kwinrc', '.kde4/share/config/'),
        ('plasma-desktop-appletsrc', '.kde4/share/config/'),
        ('plasmarc', '.kde4/share/config/'),
        ('kcmfonts', '.kde4/share/config/'),
        ('bookmarks.xml', '.kde4/share/apps/konqueror/'),
        ('favoriteapps.xml', '.kde4/share/apps/homerun/'),
        ('rekonqrc', '.kde4/share/config/'),
        ('kuriikwsfilterrc', '.kde4/share/config/'),
        ('kdeglobals', '.kde4/share/config/'),
        ('oxygenrc', '.kde4/share/config/'),
        ('yakuakerc', '.kde4/share/config/'),
        ('kickoffrc', '.kde4/share/config/'),
        ('.bashrc', ''),
        ('mimeapps.list', '.local/share/applications/'),
        ('networkmanagementrc', '.kde4/share/config/'),
        ('xdg-user-dirs-update.desktop', '.config/autostart/'),
        ('octopi-notifier.desktop', '.config/autostart/'),
        ('katerc', '.kde4/share/config/')
    ]

    for f,  d in distro_settings:
        shutil.copy2('/etc/skel/%s' % f, '%s/home/%s/%s%s' %
                     (install_path,  user,  d,  f))

    libcalamares.utils.chroot_call(
        ['chown', '-R', '%s:users' % user, "/home/%s" % user])

    print('configure kdmrc')
    kdm_conf_path = os.path.join(install_path, "usr/share/config/kdm/kdmrc")
    text = []
    with open(kdm_conf_path, "r") as kdm_conf:
        text = kdm_conf.readlines()
    with open(kdm_conf_path, "w") as kdm_conf:
        for line in text:
            if 'Theme=/usr/share/apps/kdm/themes/elarun' in line:
                line = 'Theme=/usr/share/apps/kdm/themes/midna\n'
            if '#AntiAliasing=true' in line:
                line = 'AntiAliasing=true\n'
            if '#TerminateServer=false' in line:
                line = 'TerminateServer=true\n'
            if '#HaltCmd=' in line:
                line = 'HaltCmd=/sbin/poweroff\n'
            if '#RebootCmd=' in line:
                line = 'RebootCmd=/sbin/reboot\n'
            kdm_conf.write(line)
    kdm_conf.close()

    #sddm_conf_path = os.path.join(install_path, "etc/sddm.conf")
    #text = []
    # with open(sddm_conf_path, "r") as sddm_conf:
    #     text = sddm_conf.readlines()
    #           with open(sddm_conf_path, "w") as sddm_conf:
    #     for line in text:
    #         if 'Current=maui' in line:
    #             line = 'Current=midna\n'
    #         if 'CursorTheme=' in line:
    #             line = 'CursorTheme=breeze\n'
    #     sddm_conf.write(line)
    # sddm_conf.close()

    # fix SUID to capable permissions on iputils
    libcalamares.utils.chroot_call(
        ['setcap', 'cap_net_raw=ep', '/usr/bin/ping'])
    libcalamares.utils.chroot_call(
        ['setcap', 'cap_net_raw=ep', '/usr/bin/ping6'])

    print('configure users settings done')

    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# === This file is part of Calamares - <http://github.com/calamares> ===
#
# Copyright 2014-2016, Anke Boersma <demm@kaosx.us>
#
# Calamares is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Calamares is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Calamares. If not, see <http://www.gnu.org/licenses/>.

import libcalamares
import os


def modify_grub_default(partitions, root_mount_point, distributor):

    default_dir = os.path.join(root_mount_point, "etc/default")
    default_grub = os.path.join(default_dir, "grub")
    swap_uuid = ""

    if not os.path.exists(default_dir):
        return ("Directory does not exist", "The directory {} does not exist on "
        "the target".format(default_dir))

    cryptdevice_params = []
    
    # GRUB needs to decrypt the partition that /boot is on, which may be / or /boot
    boot_mountpoint = "/"
    for partition in partitions:
        if partition["mountPoint"] == "/boot":
            boot_mountpoint = "/boot"

    for partition in partitions:
        if partition["fs"] == "linuxswap":
            swap_uuid = partition["uuid"]

        if partition["mountPoint"] == boot_mountpoint and "luksMapperName" in partition:
            cryptdevice_params = [
                "cryptdevice=UUID={!s}:{!s}".format(partition["luksUuid"],
                                                    partition["luksMapperName"]),
                "root=/dev/mapper/{!s}".format(partition["luksMapperName"])
            ]

    if swap_uuid != "":
        kernel_cmd = 'GRUB_CMDLINE_LINUX_DEFAULT="resume=UUID={!s} quiet systemd.show_status=0 {!s}"'.format(
            swap_uuid, " ".join(cryptdevice_params))
    else:
        kernel_cmd = 'GRUB_CMDLINE_LINUX_DEFAULT="quiet systemd.show_status=0 {!s}"'.format(" ".join(cryptdevice_params))

    if not os.path.exists(default_dir):
        os.mkdir(default_dir)

    with open(default_grub, 'r') as grub_file:
        lines = [x.strip() for x in grub_file.readlines()]

    for i in range(len(lines)):
        if lines[i].startswith("#GRUB_CMDLINE_LINUX_DEFAULT"):
            lines[i] = kernel_cmd
        elif lines[i].startswith("GRUB_CMDLINE_LINUX_DEFAULT"):
            lines[i] = kernel_cmd
        elif lines[i].startswith("#GRUB_DISTRIBUTOR") or lines[i].startswith("GRUB_DISTRIBUTOR"):
            lines[i] = "GRUB_DISTRIBUTOR=\"{!s}\"".format(distributor)

    if cryptdevice_params:
        lines.append("GRUB_ENABLE_CRYPTODISK=y")

    with open(default_grub, 'w') as grub_file:
        grub_file.write("\n".join(lines) + "\n")

    return None


def run():

    fw_type = libcalamares.globalstorage.value("firmwareType")
    
    if libcalamares.globalstorage.value("bootLoader") is None:
        return None
    
    if fw_type == 'efi':
        print('EFI system')
        return None

    partitions = libcalamares.globalstorage.value("partitions")
    root_mount_point = libcalamares.globalstorage.value("rootMountPoint")
    distributor = libcalamares.job.configuration["distributor"]
    return modify_grub_default(partitions, root_mount_point, distributor)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# === This file is part of Calamares - <http://github.com/calamares> ===
#
#   Copyright 2014-2026, Anke Boersma <demm@kaosx.us>
#
#   Calamares is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Calamares is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with Calamares. If not, see <http://www.gnu.org/licenses/>.

import os
import subprocess

import libcalamares


def run():
    """ Package removal module. Live only packages, surplus language packs. """

    install_path = libcalamares.globalstorage.value("rootMountPoint")
    fw_type = libcalamares.globalstorage.value("firmwareType")
    bootchoice = libcalamares.globalstorage.value("packagechooser_bootchoice")

    # remove any db.lck
    db_lock = os.path.join(install_path, "var/lib/pacman/db.lck")
    if os.path.exists(db_lock):
        with misc.raised_privileges():
            os.remove(db_lock)

    # Remove Calamares and depends
    if os.path.exists("{!s}/usr/bin/calamares" .format(install_path)):
        print('Removing installer packages')
        libcalamares.utils.target_env_call(
            ['pacman', '-Rns', '--noconfirm', 'calamares', 'calamares-debug', 'console-setup'])

    # Remove welcome
    if os.path.exists("{!s}/usr/bin/welcome" .format(install_path)):
        print('Removing live ISO packages')
        libcalamares.utils.target_env_call(
            ['pacman', '-Rns', '--noconfirm', 'welcome'])

    # Remove hardware detection
    if os.path.exists("{!s}/etc/kdeos-hwdetect.conf" .format(install_path)):
        print('Removing live start-up packages')
        libcalamares.utils.target_env_call(
            ['pacman', '-Rns', '--noconfirm', 'hardware-detection'])

    # Remove init-live & dracut-live
    if os.path.exists("{!s}/etc/live" .format(install_path)):
        print('Removing live configuration packages')
        libcalamares.utils.target_env_call(
            ['pacman', '-R', '--noconfirm', 'init-live', 'dracut-live'])

    # Remove surplus bootloader packages
    if fw_type == 'efi':
        print('Removing grub packages')
        libcalamares.utils.target_env_call(
            ['pacman', '-Rncs', '--noconfirm', 'grub'])

    if fw_type == 'bios':
        print('Removing EFI packages')
        libcalamares.utils.target_env_call(
            ['pacman', '-Rns', '--noconfirm', 'efibootmgr', 'refind-efi', 'limine'])

    if bootchoice == 'limine':
        print('Removing refind-efi packages')
        libcalamares.utils.target_env_call(
            ['pacman', '-Rns', '--noconfirm', 'refind-efi'])

    if bootchoice == 'refind':
        print('Removing limine package')
        libcalamares.utils.target_env_call(
            ['pacman', '-Rns', '--noconfirm', 'limine'])

    # Remove VirtualBox Guest packages
    if os.path.exists("{!s}/usr/bin/VBoxClient" .format(install_path)):
        if 'hypervisor' in open('/proc/cpuinfo').read():
            print('Virtual Machine')
            vbox = subprocess.check_output('systemd-detect-virt')
            if 'oracle' in str(vbox):
                print('VirtualBox')
            else:
                print('Removing guest-utils')
                libcalamares.utils.target_env_call(
                    ['pacman', '-R', '--noconfirm', 'virtualbox-guest-utils'])
        else:
            print('Removing guest-utils')
            libcalamares.utils.target_env_call(
                ['pacman', '-R', '--noconfirm', 'virtualbox-guest-utils'])

    # Remove ZFS packages
    zfs_pool_list = libcalamares.globalstorage.value("zfsPoolInfo")
    if not zfs_pool_list:
        print('ZFS not in use')
        libcalamares.utils.target_env_call(
                ['pacman', '-Rns', '--noconfirm', 'zfs-kmod'])
    else:
        print(zfs_pool_list)

    # Packagechooser outcome / remove LibreOffice l10n
    packages = libcalamares.globalstorage.value("packagechooser_packagechooserq")

    if packages == 'no_office_suite':
        print('Removing LibreOffice and Calligra')
        libcalamares.utils.target_env_call(
                ['pacman', '-Rns', '--noconfirm', 'libreoffice', 'calligra'])

    if packages == 'calligra':
        print('Calligra selected')
        libcalamares.utils.target_env_call(
            ['pacman', '-Rns', '--noconfirm', 'libreoffice'])

    if packages == 'libreoffice':
        print('LibreOffice selected')
        libcalamares.utils.target_env_call(
            ['pacman', '-Rns', '--noconfirm', 'calligra'])

    if packages == 'minimal_install':
        print('Minimal Install')
        pkgs = ['plasma-workspace-wallpapers',
                'partitionmanager',
                'gwenview',
                'spectacle',
                'sweeper',
                'plasma-firewall',
                'quassel',
                'smplayer',
                'haruna',
                'oxygen-sounds',
                'elisa',
                'kde-gtk-config',
                'tomoyo-tools',
                'kcp',
                'pacmanlogviewer',
                'seafile-client',
                'kamoso',
                'kget',
                'kompare',
                'simplescreenrecorder',
                'power-profiles-daemon',
                'k3b',
                'system-config-printer',
                'print-manager',
                'hplip',
                'epson-inkjet-printer-escpr',
                'pyqt5-python3',
                'kpat',
                'neochat',
                'libreoffice',
                'calligra']
        for pkg in pkgs:
            try:
                libcalamares.utils.target_env_call(
                    ['pacman', '-Rns', '--noconfirm', '{!s}' .format(pkg)])
            except:
                print("Could not remove package " + pkg)

    print('package removal completed')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# === This file is part of Calamares - <http://github.com/calamares> ===
#
#   Copyright 2014-2017, Anke Boersma <demm@kaosx.us>
#
#   Calamares is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Calamares is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with Calamares. If not, see <http://www.gnu.org/licenses/>.

import os
import subprocess
import shutil

import libcalamares
from libcalamares.utils import check_target_env_call


def run():
    """ Run mkinitcpio """
    
    kernel = libcalamares.job.configuration['kernel']
    check_target_env_call(['mkinitcpio', '-p', kernel])

    """ Set hardware clock """

    hwclock_rtc = ["hwclock", "--systohc", "--utc"]
    hwclock_isa = ["hwclock", "--systohc", "--utc", "--directisa"]
    is_broken_rtc = False
    is_broken_isa = False

    ret = libcalamares.utils.target_env_call(hwclock_rtc)
    if ret != 0:
        is_broken_rtc = True
        libcalamares.utils.debug("Hwclock returned error code {}".format(ret))
        libcalamares.utils.debug("  .. RTC method failed, trying ISA bus method.")
    else:
        libcalamares.utils.debug("Hwclock set using RTC method.")
    if is_broken_rtc:
        ret = libcalamares.utils.target_env_call(hwclock_isa)
        if  ret != 0:
            is_broken_isa = True
            libcalamares.utils.debug("Hwclock returned error code {}".format(ret))
            libcalamares.utils.debug("  .. ISA bus method failed.")
        else:
            libcalamares.utils.debug("Hwclock set using ISA bus methode.")
    if is_broken_rtc and is_broken_isa:
        libcalamares.utils.debug("BIOS or Kernel BUG: Setting hwclock failed.")
    
    """ Set Alsa """
    
    # setup alsa volume levels, alsa blacklist for the pc speaker, blacklist
    # for broken realtek nics
    print('setup alsa config')
    install_path = libcalamares.globalstorage.value("rootMountPoint")
    libcalamares.utils.target_env_call(
        ['/usr/bin/mkdir', '-p', '%s/etc/modprobe.d' % install_path])
    if os.path.exists('/etc/asound.state'):
        shutil.copy2('/etc/asound.state', '%s/etc/asound.state' % install_path)
    if os.path.exists('/etc/modprobe.d/alsa_blacklist.conf'):
        shutil.copy2('/etc/modprobe.d/alsa_blacklist.conf',
                     '%s/etc/modprobe.d/alsa_blacklist.conf' % install_path)
    if os.path.exists('/etc/modprobe.d/realtek_blacklist.conf'):
        shutil.copy2('/etc/modprobe.d/realtek_blacklist.conf',
                     '%s/etc/modprobe.d/realtek_blacklist.conf' % install_path)

    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# === This file is part of Calamares - <http://github.com/calamares> ===
#
#   Copyright 2014-2026, Anke Boersma <demm@kaosx.us>
#
#   Calamares is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Calamares is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with Calamares. If not, see <http://www.gnu.org/licenses/>.

import os
import shutil
import re
import subprocess

import libcalamares


def run():
    """ Create Distribution specific settings for users """
    install_path = libcalamares.globalstorage.value("rootMountPoint")
    user = libcalamares.globalstorage.value("username")

    print('create common dirs')
    common_dirs = [
        'Desktop',
        '.config',
        '.local/share/applications/',
        '.local/share/konqueror',
        '.config/autostart',
        '.config/kitty',
        '.config/niri',
        '.config/qt6ct',
        '.config/Kvantum'
    ]
    for d in common_dirs:
        libcalamares.utils.target_env_call(
            ['/usr/bin/mkdir', '-p', '/home/%s/%s' % (user,  d)])

    print('setup distribution specific settings')
    distro_settings = [
        ('bookmarks.xml', '.local/share/konqueror/'),
        ('kdeglobals', '.config/'),
        ('konsolerc', '.config/'),
        ('breezerc', '.config/'),
        ('.bashrc', ''),
        ('.xinitrc.debug', ''),
        ('mimeapps.list', '.local/share/applications/'),
        ('xdg-user-dirs-update.desktop', '.config/autostart/'),
        ('octopi-notifier.desktop', '.config/autostart/'),
        ('croeso.desktop', '.config/autostart/'),
        ('kvantum.kvconfig', '.config/Kvantum/'),
        ('dolphinrc', '.config/'),
        ('katerc', '.config/'),
        ('starship.toml', '.config/'),
        ('current-theme.conf', '.config/kitty/'),
        ('quick-access-terminal.conf', '.config/kitty/'),
        ('kitty.conf', '.config/kitty/'),
        ('config.kdl', '.config/niri/'),
        ('noctalia.kdl', '.config/niri/'),
        ('qt6ct.conf', '.config/qt6ct/'),
        ('style-colors.conf', '.config/qt6ct/')
    ]

    for f,  d in distro_settings:
        shutil.copy2('/etc/skel/%s' % f, '%s/home/%s/%s%s' %
                     (install_path,  user,  d,  f))

    shutil.copytree('/etc/skel/noctalia', '%s/home/%s/.config/noctalia' %
                     (install_path,  user))

    libcalamares.utils.target_env_call(
        ['chown', '-R', '%s:users' % user, "/home/%s" % user])

    # Packagechooser@audio outcome set to Pipewire or Pulseaudio
    audio = libcalamares.globalstorage.value("packagechooser_audio")
    if audio == 'pipewire':
        print('Pipewire selected')

    if audio == 'pulseaudio':
        print('Pulseaudio selected')
        libcalamares.utils.target_env_call(['systemctl', '--global',
                                            'disable','pipewire'])
        libcalamares.utils.target_env_call(['systemctl', '--global',
                                            'disable','pipewire-pulse'])
        libcalamares.utils.target_env_call(['systemctl', '--global',
                                            'disable','wireplumber'])

    # switch to wayland session if plasmawayland found in Live mode
    #if 'plasmawayland' in open('/etc/sddm.conf').read():
    #    print('Wayland session')
    #    sddm_conf_path = os.path.join(install_path, "etc/sddm.conf")
    #    text = []
    #    with open(sddm_conf_path, "r") as sddm_conf:
    #        text = sddm_conf.readlines()
    #        with open(sddm_conf_path, "w") as sddm_conf:
    #            for line in text:
    #                if 'Session=plasma.desktop' in line:
    #                    line = 'Session=plasmawayland.desktop\n'
    #                sddm_conf.write(line)
    #    sddm_conf.close()

    # fix SUID to capable permissions on iputils
    libcalamares.utils.target_env_call(
        ['setcap', 'cap_net_raw=ep', '/usr/bin/ping'])

    # set pacman.conf for kf5 needed repos
    #shutil.copy2('/etc/skel/pacman.conf', '%s/etc/pacman.conf' % install_path)

    # aquire ISO version for sysinfo call
    label = ""
    path = os.path.join(install_path, "etc/KaOS-release")
    p = subprocess.Popen('blkid', shell=True, stdout=subprocess.PIPE)
    for line in p.stdout.readlines():
        s = line.decode('ascii')
        s = s.rstrip('\n')
        label +=s

    print(label)
    if 'plasma' in open('/etc/sddm.conf').read():
        m = re.search("KAOS_\d{8}", label);
        print (m.group(0))
        lines = (m.group(0))
    else:
        m = re.search("KAOS_\d{8}", label);
        print (m.group(0))
        lines = (m.group(0))

    if os.path.exists(path):
        with open(path, 'w') as f:
            for l in lines:
                f.write(l)
    f.close()


    print('configure users settings done')

    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# === This file is part of Calamares - <http://github.com/calamares> ===
#
#   Copyright 2014, Philip Müller <philm@manjaro.org>
#   Copyright 2014, Teo Mrnjavac <teo@kde.org>
#   Copyright 2014-2026, Anke Boersma <demm@kaosx.us>
#
#   Calamares is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Calamares is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with Calamares. If not, see <http://www.gnu.org/licenses/>.

import os
import collections
import re
import libcalamares
import subprocess


DesktopEnvironment = collections.namedtuple('DesktopEnvironment', ['executable', 'desktop_file'])

desktop_environments = [
    DesktopEnvironment('/usr/bin/startplasma-x11', 'plasmax11'), # Plasma with X
    DesktopEnvironment('/usr/bin/startkde-wayland', 'plasma'), # Plasma with Wayland
    DesktopEnvironment('/usr/bin/niri', 'niri'), # niri session
]

def find_desktop_environment(root_mount_point):
    for desktop_environment in desktop_environments:
        if os.path.exists('%s%s' % (root_mount_point, desktop_environment.executable)) \
           and os.path.exists('%s/usr/share/wayland-sessions/%s.desktop' % (root_mount_point, desktop_environment.desktop_file)):
            return desktop_environment
    return None

def set_autologin(username, displaymanagers, root_mount_point):
    """ Enables automatic login for the installed desktop managers """

    if "sddm" in displaymanagers:
        # Systems with Sddm as Desktop Manager
        sddm_conf_path = os.path.join(root_mount_point, "etc/sddm.conf")
        if os.path.isfile(sddm_conf_path):
            print('SDDM config file exists')
        else:
            libcalamares.utils.check_target_env_call(["sh", "-c", "sddm --example-config > /etc/sddm.conf"])
        text = []
        with open(sddm_conf_path, 'r') as sddm_conf:
            text = sddm_conf.readlines()
        with open(sddm_conf_path, 'w') as sddm_conf:
            for line in text:
                # User= line, possibly commented out
                if re.match('\\s*(?:#\\s*)?User=', line):
                    line = 'User={}\n'.format(username)
                # Session= line, commented out or with empty value
                if re.match('\\s*#\\s*Session=|\\s*Session=$', line):
                    default_desktop_environment = find_desktop_environment(root_mount_point)
                    if default_desktop_environment != None:
                        line = 'Session={}.desktop\n'.format(default_desktop_environment.desktop_file)
                sddm_conf.write(line)

    return None

def run():
    """ Configure display managers """
    # This module will try to set up (including autologin) a displaymanager.
    # Most distros will probably only ship one displaymanager.
    # If a displaymanager is in the list but not installed, this module quits with error.

    if "displaymanagers" in libcalamares.job.configuration:
        displaymanagers = libcalamares.job.configuration["displaymanagers"]

    if libcalamares.globalstorage.contains("displaymanagers"):
        displaymanagers = libcalamares.globalstorage.value("displaymanagers")

    if displaymanagers is None:
        return "No display managers selected for the displaymanager module.",\
               "The displaymanagers list is empty or undefined in both globalstorage and displaymanager.conf."

    username = libcalamares.globalstorage.value("autoLoginUser")
    root_mount_point = libcalamares.globalstorage.value("rootMountPoint")

    # Setup sddm
    if "sddm" in displaymanagers:
        if not os.path.exists("%s/usr/bin/sddm" % root_mount_point):
            return "sddm selected but not installed", ""

    if username != None:
        libcalamares.utils.debug(
            "Setting up autologin for user %s." % username)
        return set_autologin(username, displaymanagers, root_mount_point)

    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# === This file is part of Calamares - <http://github.com/calamares> ===
#
#   Copyright 2014, Anke Boersma <demm@kaosx.us>
#   Copyright 2014, Benjamin Vaudour <benjamin.vaudour@yahoo.fr>
#
#   Calamares is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Calamares is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with Calamares. If not, see <http://www.gnu.org/licenses/>.

import libcalamares
import os
 
install_path = libcalamares.globalstorage.value("rootMountPoint")
menu_entries = []
 
def read_grub():
    global menu_entries
    grub_path = ""
    if os.path.exists('%s/boot/grub/grub.cfg' % install_path):
        grub_path = os.path.join(install_path, 'boot', 'grub', 'grub.cfg')
    elif os.path.exists('%s/boot/grub2/grub.cfg' % install_path):
        grub_path = os.path.join(install_path, 'boot', 'grub2', 'grub.cfg')
    elif os.path.exists('%s/boot/efi/grub/grub.cfg' % install_path):
        grub_path = os.path.join(install_path, 'boot', 'efi', 'grub', 'grub.cfg')
    elif os.path.exists('%s/boot/efi/grub2/grub.cfg' % install_path):
        grub_path = os.path.join(install_path, 'boot', 'efi', 'grub2', 'grub.cfg')
    if not grub_path:
        return
    o = False
    with open(grub_path, 'r') as f:
        for l in f:
            if l.strip()[:9] == 'menuentry' and l[-2] == '{':
                o = True
                e = {}
                e["title"] = l[11:19]
            elif l.lstrip()[:5] == 'linux':
                l = l.lstrip()[11:].lstrip()
                i = l.find(' ')
                e["linux"] = l[:i]
                e["options"] = l[i+1:].rstrip('\n')
            elif l.lstrip()[:6] == 'initrd':
                e["initrd"] = l.lstrip()[12:].strip().rstrip('\n')
            elif '}' in l and o:
                o = False
                menu_entries.append(e)
                break
    f.close() 
 
# Write the menu entry .conf  
def write_conf(e):
    lines = [
            '## This is just an example config file.\n',
            '## Please edit the paths and kernel parameters according to your system.\n',
            '\n',
            'title   %s\n' % e["title"],
            'linux   %s\n' % e["linux"],
            'initrd  %s\n' % e["initrd"],
            'options %s\n' % e["options"],
        ]
    path = os.path.join(install_path, "boot", "loader", "entries", "%s.conf" % e["title"])
    with open(path, 'w') as f:
        for l in lines:
            f.write(l)
    f.close()
 
def run():
    fw_type = libcalamares.globalstorage.value("firmwareType")
    if fw_type == 'efi':
        read_grub()
        print(menu_entries)
        for e in menu_entries:
            write_conf(e)
 
run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# === This file is part of Calamares - <http://github.com/calamares> ===
#
#   Copyright 2014-2018 KaOS (http://kaosx.us)
#
#   Calamares is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Calamares is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with Calamares. If not, see <http://www.gnu.org/licenses/>.
#  MA 02110-1301, USA.

import os
import shutil
import subprocess
import re

import libcalamares


def run():
    """ Clean up unused drivers """

    print('cleaning up video drivers')

    install_path = libcalamares.globalstorage.value("rootMountPoint")

    # remove any db.lck
    db_lock = os.path.join(install_path, "var/lib/pacman/db.lck")
    if os.path.exists(db_lock):
        with misc.raised_privileges():
            os.remove(db_lock)

    searchf = "/var/log/used_gfx"
    f = open("/var/log/used_gfx", "r")
    file_c = f.read()
    
    if os.path.exists(searchf):
        if "i915" in file_c:
            libcalamares.utils.debug(file_c)
        else:
            try:
                libcalamares.utils.target_env_call(['pacman', '-R', '--noconfirm',
                                                'xf86-video-intel'])
            except Exception as e:
                pass
        if "nouveau" in file_c:
            libcalamares.utils.debug(file_c)
            sddm_conf_path = os.path.join(install_path, "etc/sddm.conf")
            text = []
            with open(sddm_conf_path, 'r') as sddm_conf:
                text = sddm_conf.readlines()
            with open(sddm_conf_path, 'w') as sddm_conf:
                for line in text:
                    if re.match('Session=plasmawayland.desktop', line):
                        line = 'Session=plasma.desktop'
                    sddm_conf.write(line)                    
        else:
            try:
                libcalamares.utils.target_env_call(['pacman', '-R', '--noconfirm',
                                                'xf86-video-nouveau'])
            except Exception as e:
                pass
        if "amdgpu" in file_c:
            libcalamares.utils.debug(file_c)
        else:
            try:
                libcalamares.utils.target_env_call(['pacman', '-R', '--noconfirm',
                                                'xf86-video-amdgpu'])
            except Exception as e:
                pass
        if "ati" in file_c or "radeon" in file_c:
            libcalamares.utils.debug(file_c)
        else:
            try:
                libcalamares.utils.target_env_call(['pacman', '-R', '--noconfirm',
                                                'xf86-video-ati'])
            except Exception as e:
                pass
        if "vmware" in file_c or "vmwgfx" in file_c:
            libcalamares.utils.debug(file_c)
        else:
            try:
                libcalamares.utils.target_env_call(['pacman', '-R', '--noconfirm',
                                                'xf86-video-vmware'])
            except Exception as e:
                pass
        f.close()
    else:
        try:
            libcalamares.utils.target_env_call(['pacman', '-R', '--noconfirm', 'xf86-video-ati',
                                            'xf86-video-vmware', 'xf86-video-amdgpu'])
        except Exception as e:
            pass

    print('video driver removal complete')

    print('cleaning up input drivers')

    xorg = open("/var/log/Xorg.0.log",  errors="surrogateescape").read()
    if 'wacom' in xorg:
        print('wacom in use')
    else:
        try:
            libcalamares.utils.target_env_call(['pacman', '-Rncs', '--noconfirm',
                                            'xf86-input-wacom'])
        except Exception as e:
            pass

    print('input driver removal complete')

    print('job_cleanup_drivers')

    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# === This file is part of Calamares - <http://github.com/calamares> ===
#
#   Copyright 2014, Anke Boersma <demm@kaosx.us>
#   Copyright 2014, Philip Müller <philm@manjaro.org>
#   Copyright 2014, Teo Mrnjavac <teo@kde.org>
#
#   Calamares is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Calamares is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with Calamares. If not, see <http://www.gnu.org/licenses/>.

import libcalamares


def run():
    """ Setup systemd services """

    services = libcalamares.job.configuration['services']
    targets = libcalamares.job.configuration['targets']
    disable = libcalamares.job.configuration['disable']

    # enable services
    for svc in services:
        ec = libcalamares.utils.target_env_call(['systemctl',
                                             'enable',
                                             '{}.service'.format(svc['name'])])
        if ec != 0:
            if svc['mandatory']:
                return "Cannot enable systemd service {}".format(svc['name']),\
                       "systemctl enable call in chroot returned error code {}".format(ec)
            else:
                libcalamares.utils.debug(
                    "Cannot enable systemd service {}".format(svc['name']))
                libcalamares.utils.debug(
                    "systemctl enable call in chroot returned error code {}".format(ec))

    # enable targets
    for tgt in targets:
        ec = libcalamares.utils.target_env_call(['systemctl',
                                             'enable',
                                             '{}.target'.format(tgt['name'])])
        if ec != 0:
            if tgt['mandatory']:
                return "Cannot enable systemd target {}".format(tgt['name']), \
                       "systemctl enable call in chroot returned error code {}".format(ec)
            else:
                libcalamares.utils.debug(
                    "Cannot enable systemd target {}".format(tgt['name']))
                libcalamares.utils.debug(
                    "systemctl enable call in chroot returned error code {}".format(ec))

    for dbl in disable:
        ec = libcalamares.utils.target_env_call(['systemctl', 'disable', '{}.service'.format(dbl['name'])])

        if ec != 0:
            if dbl['mandatory']:
                return "Cannot disable systemd service {}".format(dbl['name']), \
                       "systemctl disable call in chroot returned error code {}".format(ec)
            else:
                libcalamares.utils.debug("Cannot disable systemd service {}".format(dbl['name']))
                libcalamares.utils.debug("systemctl disable call in chroot returned error code {}".format(ec))

    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# === This file is part of Calamares - <https://calamares.io> ===
#
#   SPDX-FileCopyrightText: 2014 Philip Müller <philm@manjaro.org>
#   SPDX-FileCopyrightText: 2014 Teo Mrnjavac <teo@kde.org>
#   SPDX-FileCopyrightText: 2017 Alf Gaida <agaida@siduction.org>
#   SPDX-FileCopyrightText: 2019 Adriaan de Groot <groot@kde.org>
#   SPDX-FileCopyrightText: 2021 Anke boersma <demm@koasx.us>
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   Calamares is Free Software: see the License-Identifier above.
#

import os
import shutil

import libcalamares

import gettext
_ = gettext.translation("calamares-python",
                        localedir=libcalamares.utils.gettext_path(),
                        languages=libcalamares.utils.gettext_languages(),
                        fallback=True).gettext


def pretty_name():
    return _("Saving network configuration.")


def run():
    """
    Setup network configuration
    """
    root_mount_point = libcalamares.globalstorage.value("rootMountPoint")
    user = libcalamares.globalstorage.value("username")
    live_user = os.getlogin()

    if root_mount_point is None:
        libcalamares.utils.warning("rootMountPoint is empty, {!s}".format(root_mount_point))
        return (_("Configuration Error"),
                _("No root mount point is given for <pre>{!s}</pre> to use." ).format("networkcfg"))

    source_nm = "/etc/NetworkManager/system-connections/"
    target_nm = os.path.join(
        root_mount_point, "etc/NetworkManager/system-connections/"
        )

    # Sanity checks.  We don't want to do anything if a network
    # configuration already exists on the target
    if os.path.exists(source_nm) and os.path.exists(target_nm):
        for network in os.listdir(source_nm):
            # Skip LTSP live
            if network == "LTSP":
                continue

            source_network = os.path.join(source_nm, network)
            target_network = os.path.join(target_nm, network)

            if os.path.exists(target_network):
                continue

            try:
                shutil.copy(source_network, target_network, follow_symlinks=False)
                if live_user in open(target_network).read():
                    text = []
                    with open(target_network, "r", encoding="UTF-8") as network_conf:
                        text = network_conf.readlines()
                        with open(target_network, "w", encoding="UTF-8") as network_conf:
                            for line in text:
                                if 'permissions=user:{}:;'.format(live_user) in line:
                                    line = 'permissions=user:{}:;\n'.format(user)
                                network_conf.write(line)
                    network_conf.close()
            except FileNotFoundError:
                libcalamares.utils.debug(
                    "Can't copy network configuration files in "
                    + "{}".format(source_network)
                    )
            except FileExistsError:
                pass

    # We need to overwrite the default resolv.conf in the chroot.
    source_resolv = "/etc/resolv.conf"
    target_resolv = os.path.join(root_mount_point, "etc/resolv.conf")
    if source_resolv != target_resolv and os.path.exists(source_resolv):
        try:
            os.remove(target_resolv)
        except Exception as err:
            libcalamares.utils.debug(
                "Couldn't remove {}: {}".format(target_resolv, err)
                )

        try:
            shutil.copy(source_resolv, target_resolv, follow_symlinks=False)
        except Exception as err:
            libcalamares.utils.debug(
                "Can't copy resolv.conf from {}: {}".format(source_resolv, err)
                )

    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# === This file is part of Calamares - <https://calamares.io> ===
#
#   SPDX-FileCopyrightText: 2025 Anke Boersma <demm@kaosx.us>
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   Calamares is Free Software: see the License-Identifier above.
#

import libcalamares

import os
import subprocess

from libcalamares.utils import check_target_env_call

import gettext
_ = gettext.translation("calamares-python",
                        localedir=libcalamares.utils.gettext_path(),
                        languages=libcalamares.utils.gettext_languages(),
                        fallback=True).gettext


def pretty_name():
    return _("Install Limine Bootloader.")

def get_uuid():
    root_mount_point = libcalamares.globalstorage.value("rootMountPoint")
    print(root_mount_point)
    partitions = libcalamares.globalstorage.value("partitions")
    print(partitions)
    for partition in partitions:
        if partition["mountPoint"] == "/":
            libcalamares.utils.debug(partition["uuid"])
            return partition["uuid"]
    return ""

def get_zfs_root():
    """
    Looks in global storage to find the zfs root

    :return: A string containing the path to the zfs root or None if it is not found
    """

    zfs = libcalamares.globalstorage.value("zfsDatasets")

    if not zfs:
        libcalamares.utils.warning("Failed to locate zfs dataset list")
        return None

    # Find the root dataset
    for dataset in zfs:
        try:
            if dataset["mountpoint"] == "/":
                return dataset["zpool"] + "/" + dataset["dsName"]
        except KeyError:
            # This should be impossible
            libcalamares.utils.warning("Internal error handling zfs dataset")
            raise

    return None

def is_zfs_root(partition):
    """ Returns True if the partition object refers to a zfs root filesystem

    :param partition: A partition map from global storage
    :return: True if zfs and root, False otherwise
    """
    return partition["mountPoint"] == "/" and partition["fs"] == "zfs"


def create_conf(uuid, conf_path):
    #timeout = libcalamares.job.configuration["timeout"]
    #distribution = libcalamares.job.configuration["distribution"]
    #protocol = libcalamares.job.configuration["protocol"]
    #module_path = libcalamares.job.configuration["module_path"]
    #fallback = libcalamares.job.configuration["fallback"]
    partitions = libcalamares.globalstorage.value("partitions")

    kernel_cmdline = ["quiet systemd.show_status=0"]
    swap = ""
    swap_luks = ""
    cryptdevice_params = []
    btrfs_params = ""
    zfs_params = ""

    for partition in partitions:
        if partition["fs"] == "linuxswap" and not "luksMapperName" in partition:
            swap = partition["uuid"]

        if partition["fs"] == "linuxswap" and "luksMapperName" in partition:
            swap_luks = partition["luksMapperName"]

        if partition["mountPoint"] == "/" and "luksMapperName" in partition:
            cryptdevice_params = [
                "cryptdevice=UUID={!s}:{!s}".format(partition["luksUuid"],
                                                    partition["luksMapperName"]),
                "root=/dev/mapper/{!s}".format(partition["luksMapperName"]),
                "resume=/dev/mapper/{!s}".format(partition["luksMapperName"])
            ]

        # Limine with a BTRFS root filesystem needs to be told
        # about the root subvolume.
        if partition["mountPoint"] == "/" and partition["fs"] == "btrfs":
            btrfs_params = "rootflags=subvol=@"

        # zfs needs to be told the location of the root dataset
        if is_zfs_root(partition):
            zfs_root_path = get_zfs_root()
            if zfs_root_path is not None:
                zfs_params = ("root=ZFS=" + zfs_root_path)
            else:
                # Something is really broken if we get to this point
                libcalamares.utils.warning("Internal error handling zfs dataset")
                raise Exception("Internal zfs data missing, please contact your distribution")

    if cryptdevice_params:
        kernel_cmdline.extend(cryptdevice_params)
    else:
        if zfs_params:
            kernel_cmdline.append(zfs_params)
        else:
            kernel_cmdline.append("root=UUID={!s}".format(uuid))

    if swap:
        kernel_cmdline.append("resume=UUID={!s}".format(swap))
    if swap_luks:
        kernel_cmdline.append("resume=/dev/mapper/{!s}".format(swap_luks))
    if btrfs_params:
        print('BTRFS')
        kernel_cmdline.append(btrfs_params)

    lines = [
        'timeout 10\n',
        'default_entry: 1\n',
        'interface_branding_colour: 1\n',
        'remember_last_entry: yes\n',
        'term_background: CC000000\n',
        'term_foreground: FFFFFF\n',
        'term_margin: 100\n',
        'wallpaper: boot():/limine/wallpaper.jpg\n\n',
        '/KaOS 2026.03\n',
        '    comment: KaOS GNU/Linux, with Linux kernel\n',
        '    protocol: linux\n',
        '    kernel_path: boot():/vmlinuz-linux\n',
        '    module_path: boot():/initramfs-linux.img\n',
        '    kernel_cmdline: {!s} rw\n\n'.format(" ".join(kernel_cmdline)),
        '',
        '/KaOS 2026.03 Fallback\n',
        '    comment: KaOS GNU/Linux, with Linux kernel, fallback\n',
        '    protocol: linux\n'
        '    kernel_path: boot():/vmlinuz-linux\n',
        '    module_path: boot():/initramfs-linux-fallback.img\n',
        '    kernel_cmdline: {!s} rw\n'.format(" ".join(kernel_cmdline)),
    ]

    with open(conf_path, 'w') as f:
        for l in lines:
            f.write(l)
    f.close()


def install_limine():
    install_path = libcalamares.globalstorage.value("rootMountPoint")
    uuid = get_uuid()
    conf_path = os.path.join(
        install_path, "boot", "limine", "limine.conf")
    partitions = libcalamares.globalstorage.value("partitions")
    device = ""

    for partition in partitions:
        if partition["mountPoint"] == "/boot":
            libcalamares.utils.debug(partition["device"])
            boot_device = partition["device"]
            boot_p = boot_device[-1:]
            device = boot_device[:-1]
            if boot_device.startswith('/dev/nvme'):
                device = boot_device[:-2]

            if not boot_p or not device:
                return ("EFI directory /boot not found!",
                        "Boot partition: \"{!s}\"",
                        "Boot device: \"{!s}\"".format(boot_p, device))
            else:
                print("EFI directory: /boot")
                print("Boot partition: \"{!s}\"".format(boot_p))
                print("Boot device: \"{!s}\"".format(device))

    if not device:
        print("WARNING: no EFI system partition or EFI system partition mount point not set.")
        print("         >>> no EFI bootloader will be installed <<<")
        return None
    print("Set 'EF00' flag")
    subprocess.call(
        ["sgdisk", "--typecode={!s}:EF00".format(boot_p), "{!s}".format(device)])
    cmd = [
        "efibootmgr",
        "--create",
        "--disk", f"{device}",
        "--part", f"{boot_p}",
        "--label", "KaOS with Limine Bootloader",
        "--loader", "/limine/BOOTX64.EFI"
    ]
    subprocess.call(cmd)
    create_conf(uuid, conf_path)


def run():
    #boot_loader = libcalamares.globalstorage.value("bootLoader")
    # Packagechooser@bootchoice outcome
    bootchoice = libcalamares.globalstorage.value("packagechooser_bootchoice")

    if bootchoice != 'limine':
        return None

    install_limine()
    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# === This file is part of Calamares - <https://calamares.io> ===
#
#   SPDX-FileCopyrightText: 2021 -2022 Anke Boersma <demm@kaosx.us>
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   Calamares is Free Software: see the License-Identifier above.
#

import libcalamares

import os
import subprocess

from libcalamares.utils import check_target_env_call

import gettext
_ = gettext.translation("calamares-python",
                        localedir=libcalamares.utils.gettext_path(),
                        languages=libcalamares.utils.gettext_languages(),
                        fallback=True).gettext


def pretty_name():
    return _("Install rEFInd.")

def get_uuid():
    root_mount_point = libcalamares.globalstorage.value("rootMountPoint")
    print(root_mount_point)
    partitions = libcalamares.globalstorage.value("partitions")
    print(partitions)
    for partition in partitions:
        if partition["mountPoint"] == "/":
            libcalamares.utils.debug(partition["uuid"])
            return partition["uuid"]
    return None

def get_zfs_root():
    """
    Looks in global storage to find the zfs root

    :return: A string containing the path to the zfs root or None if it is not found
    """

    zfs = libcalamares.globalstorage.value("zfsDatasets")

    if not zfs:
        libcalamares.utils.warning("Failed to locate zfs dataset list")
        return None

    # Find the root dataset
    for dataset in zfs:
        try:
            if dataset["mountpoint"] == "/":
                return dataset["zpool"] + "/" + dataset["dsName"]
        except KeyError:
            # This should be impossible
            libcalamares.utils.warning("Internal error handling zfs dataset")
            raise

    return None

def is_zfs_root(partition):
    """ Returns True if the partition object refers to a zfs root filesystem

    :param partition: A partition map from global storage
    :return: True if zfs and root, False otherwise
    """
    return partition["mountPoint"] == "/" and partition["fs"] == "zfs"

def update_conf(uuid, conf_path):
    """
    Updates the created rEFInd configuration file based on given parameters.
    """
    partitions = libcalamares.globalstorage.value("partitions")

    kernel_params = ["quiet systemd.show_status=0"]
    swap = ""
    swap_luks = ""
    cryptdevice_params = []
    btrfs_params = ""
    zfs_params = ""

    for partition in partitions:
        if partition["fs"] == "linuxswap" and not "luksMapperName" in partition:
            swap = partition["uuid"]

        if partition["fs"] == "linuxswap" and "luksMapperName" in partition:
            swap_luks = partition["luksMapperName"]

        if partition["mountPoint"] == "/" and "luksMapperName" in partition:
            cryptdevice_params = [
                "cryptdevice=UUID={!s}:{!s}".format(partition["luksUuid"],
                                                    partition["luksMapperName"]),
                "root=/dev/mapper/{!s}".format(partition["luksMapperName"]),
                "resume=/dev/mapper/{!s}".format(partition["luksMapperName"])
            ]

        # rEFInd with a BTRFS root filesystem needs to be told
        # about the root subvolume.
        if partition["mountPoint"] == "/" and partition["fs"] == "btrfs":
            btrfs_params = "rootflags=subvol=@"

        # zfs needs to be told the location of the root dataset
        if is_zfs_root(partition):
            zfs_root_path = get_zfs_root()
            if zfs_root_path is not None:
                zfs_params = ("zfs=bootfs root=ZFS=" + zfs_root_path)
            else:
                # Something is really broken if we get to this point
                libcalamares.utils.warning("Internal error handling zfs dataset")
                raise Exception("Internal zfs data missing, please contact your distribution")

    if cryptdevice_params:
        kernel_params.extend(cryptdevice_params)
    else:
        if zfs_params:
            kernel_params.append(zfs_params)
        else:
            kernel_params.append("root=UUID={!s}".format(uuid))

    if swap:
        kernel_params.append("resume=UUID={!s}".format(swap))
    if swap_luks:
        kernel_params.append("resume=/dev/mapper/{!s}".format(swap_luks))
    if btrfs_params:
        print('BTRFS')
        kernel_params.append(btrfs_params)

    with open(conf_path, "r") as refind_file:
        filedata = [x.strip() for x in refind_file.readlines()]

    with open(conf_path, 'w') as refind_file:
        for line in filedata:
            if line.startswith('"Boot with standard options"'):
                line = '"Boot with standard options"    "rw {!s}"'.format(" ".join(kernel_params))
            refind_file.write(line + "\n")
    refind_file.close()


def install_refind():
    install_path = libcalamares.globalstorage.value("rootMountPoint")
    uuid = get_uuid()
    conf_path = os.path.join(install_path, "boot/refind_linux.conf")
    partitions = libcalamares.globalstorage.value("partitions")
    device = ""

    for partition in partitions:
        if partition["mountPoint"] == "/boot":
            libcalamares.utils.debug(partition["device"])
            boot_device = partition["device"]
            boot_p = boot_device[-1:]
            device = boot_device[:-1]
            if boot_device.startswith('/dev/nvme'):
                device = boot_device[:-2]

            if not boot_p or not device:
                return ("EFI directory /boot not found!",
                        "Boot partition: \"{!s}\"",
                        "Boot device: \"{!s}\"".format(boot_p, device))
            else:
                print("EFI directory: /boot")
                print("Boot partition: \"{!s}\"".format(boot_p))
                print("Boot device: \"{!s}\"".format(device))

    if not device:
        print("WARNING: no EFI system partition or EFI system partition mount point not set.")
        print("         >>> no EFI bootloader will be installed <<<")
        return None
    print("Set 'EF00' flag")
    subprocess.call(
        ["sgdisk", "--typecode={!s}:EF00".format(boot_p), "{!s}".format(device)])
    subprocess.call(
        ["refind-install", "--root", "{!s}".format(install_path)])
    update_conf(uuid, conf_path)

def run():
    """
    Optional entry for when providing bootloader choices.
    Values taken from a packagechooser instance.
    Module won't run, if value not present.
    """
    #boot_loader = libcalamares.globalstorage.value("bootLoader")
    bootchoice = libcalamares.globalstorage.value("packagechooser_bootchoice")

    if bootchoice != "refind":
        return None

    install_refind()
    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# === This file is part of Calamares - <https://calamares.io> ===
#
#   SPDX-FileCopyrightText: 2014-2021 Anke Boersma <demm@kaosx.us>
#   SPDX-FileCopyrightText: 2014, Benjamin Vaudour <benjamin.vaudour@yahoo.fr>
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   Calamares is Free Software: see the License-Identifier above.
#

import libcalamares

import os
import subprocess

from libcalamares.utils import check_target_env_call


def detect_firmware_type():
    # Check for EFI variables support
    if(os.path.exists("/sys/firmware/efi/efivars")):
        fw_type = 'efi'
    else:
        fw_type = 'bios'

    libcalamares.globalstorage.insert("firmwareType", fw_type)
    libcalamares.utils.debug("Firmware type: {!s}".format(fw_type))


def get_uuid():
    root_mount_point = libcalamares.globalstorage.value("rootMountPoint")
    print(root_mount_point)
    partitions = libcalamares.globalstorage.value("partitions")
    print(partitions)
    for partition in partitions:
        if partition["mountPoint"] == "/":
            libcalamares.utils.debug(partition["uuid"])
            return partition["uuid"]
    return ""


def install_grub(boot_loader):
    install_path = boot_loader["installPath"]
    check_target_env_call(["grub-install", install_path])
    check_target_env_call(["grub-mkconfig", "-o", "/boot/grub/grub.cfg"])


def run():
    fw_type = libcalamares.globalstorage.value("firmwareType")
    boot_loader = libcalamares.globalstorage.value("bootLoader")
    # Packagechooser@bootchoice outcome
    bootchoice = libcalamares.globalstorage.value("packagechooser_bootchoice")

    if bootchoice != 'grub':
        return None

    if libcalamares.globalstorage.value("bootLoader") is None and fw_type != "efi":
        libcalamares.utils.debug('no bootloader install')
        return None

    install_grub(boot_loader)
    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# === This file is part of Calamares - <https://calamares.io> ===
#
#   SPDX-FileCopyrightText: 2014 Aurélien Gâteau <agateau@kde.org>
#   SPDX-FileCopyrightText: 2016 Anke Boersma <demm@kaosx.us>
#   SPDX-FileCopyrightText: 2018 Adriaan de Groot <groot@kde.org>
#   SPDX-License-Identifier: GPL-3.0-or-later
#
#   Calamares is Free Software: see the License-Identifier above.
#

import os
import subprocess
import shutil

import libcalamares
from libcalamares.utils import gettext_path, gettext_languages

import gettext
_translation = gettext.translation("calamares-python",
                                   localedir=gettext_path(),
                                   languages=gettext_languages(),
                                   fallback=True)
_ = _translation.gettext
_n = _translation.ngettext


def pretty_name():
    return _( "Unmount file systems." )


def list_mounts(root_mount_point):
    """ List mount points.

    :param root_mount_point:
    :return:
    """
    lst = []

    root_mount_point = os.path.normpath(root_mount_point)
    for line in open("/etc/mtab").readlines():
        device, mount_point, _ = line.split(" ", 2)

        if os.path.commonprefix([root_mount_point, mount_point]) == root_mount_point:
            lst.append((device, mount_point))

    return lst


def export_zpools(root_mount_point):
    """ Exports the zpools if defined in global storage

    :param root_mount_point: The absolute path to the root of the install
    :return:
    """
    try:
        zfs_pool_list = libcalamares.globalstorage.value("zfsPoolInfo")
        zfs_pool_list.sort(reverse=True, key=lambda x: x["poolName"])
        if zfs_pool_list:
            for zfs_pool in zfs_pool_list:
                try:
                    libcalamares.utils.host_env_process_output(['zpool', 'export', zfs_pool["poolName"]])
                except subprocess.CalledProcessError:
                    libcalamares.utils.warning("Failed to export zpool")
    except Exception as e:
        # If this fails it shouldn't cause the installation to fail
        libcalamares.utils.warning("Received exception while exporting zpools: " + format(e))
        pass


def run():
    """ Unmounts given mountpoints in decreasing order.

    :return:
    """
    root_mount_point = libcalamares.globalstorage.value("rootMountPoint")

    if(libcalamares.job.configuration and
       "srcLog" in libcalamares.job.configuration and
       "destLog" in libcalamares.job.configuration):
        log_source = libcalamares.job.configuration["srcLog"]
        log_destination = libcalamares.job.configuration["destLog"]
        # Relocate log_destination into target system
        log_destination = '{!s}/{!s}'.format(root_mount_point, log_destination)
        # Make sure source is a string
        log_source = '{!s}'.format(log_source)

        # copy installation log before umount
        if os.path.exists(log_source):
            try:
                shutil.copy2(log_source, log_destination)
            except Exception as e:
                libcalamares.utils.warning("Could not preserve file {!s}, "
                                       "error {!s}".format(log_source, e))

    if not root_mount_point:
        return ("No mount point for root partition in globalstorage",
                "globalstorage does not contain a \"rootMountPoint\" key, "
                "doing nothing")

    if not os.path.exists(root_mount_point):
        return ("Bad mount point for root partition in globalstorage",
                "globalstorage[\"rootMountPoint\"] is \"{}\", which does not "
                "exist, doing nothing".format(root_mount_point))

    lst = list_mounts(root_mount_point)
    # Sort the list by mount point in decreasing order. This way we can be sure
    # we unmount deeper dirs first.
    lst.sort(key=lambda x: x[1], reverse=True)

    for device, mount_point in lst:
        # On success, no output; if the command fails, its output is
        # in the exception object.
        subprocess.check_output(["umount", "-lv", mount_point], stderr=subprocess.STDOUT)

    export_zpools(root_mount_point)

    os.rmdir(root_mount_point)

    return None



#!/usr/bin/env python3
# encoding: utf-8
# === This file is part of Calamares - <http://github.com/calamares> ===
#
#   Copyright 2014, Teo Mrnjavac <teo@kde.org>
#
#   Calamares is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Calamares is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with Calamares. If not, see <http://www.gnu.org/licenses/>.

import os
import re
import shutil
import subprocess
import sys
import tempfile
from collections import namedtuple

from libcalamares import *

class UnpackEntry:
    __slots__= ['source', 'sourcefs', 'destination', 'copied', 'total']

    def __init__(self, source, sourcefs, destination):
        self.source = source
        self.sourcefs = sourcefs
        self.destination = destination
        self.copied = 0
        self.total = 0

ON_POSIX = 'posix' in sys.builtin_module_names


def file_copy(source, dest, progress_cb):
    # Environment used for executing rsync properly
    # Setting locale to C (fix issue with tr_TR locale)
    at_env = os.environ
    at_env["LC_ALL"] = "C"

    # `source` *must* end with '/' otherwise a directory named after the source
    # will be created in `dest`: ie if `source` is "/foo/bar" and `dest` is
    # "/dest", then files will be copied in "/dest/bar".
    source += "/"

    process = subprocess.Popen(['rsync', '-ar', '--progress', source, dest],
                               env=at_env,
                               bufsize=1,
                               stdout=subprocess.PIPE,
                               close_fds=ON_POSIX)

    for line in iter(process.stdout.readline, b''):
        # small comment on this regexp.
        # rsync outputs three parameters in the progress.
        # xfer#x => i try to interpret it as 'file copy try no. x'
        # to-check=x/y, where:
        #  - x = number of files yet to be checked
        #  - y = currently calculated total number of files.
        # but if you're copying directory with some links in it, the xfer#
        # might not be a reliable counter (for one increase of xfer, many
        # files may be created).
        # In case of manjaro, we pre-compute the total number of files.
        # therefore we can easily subtract x from y in order to get real files
        # copied / processed count.
        m = re.findall(r'xfr#(\d+), ir-chk=(\d+)/(\d+)', line.decode())
        if m:
            # we've got a percentage update
            num_files_remaining = int(m[0][1])
            num_files_total_local = int(m[0][2])
            # adjusting the offset so that progressbar can be continuesly drawn
            num_files_copied = num_files_total_local - num_files_remaining

            # I guess we're updating every 100 files...
            if num_files_copied % 100 == 0:
                progress_cb(num_files_copied)
    process.wait()
    if process.returncode != 0:
        return "rsync failed with error code {}.".format(process.returncode)
    return None


class UnpackOperation:

    def __init__(self, entries):
        self.entries = entries
        self.entry_for_source = dict((x.source, x) for x in self.entries)

    def report_progress(self):
        progress = float(0)
        for entry in self.entries:
            if entry.total == 0:
                continue

            partialprogress = 0.05  # Having a total !=0 gives 5%

            partialprogress += 0.95 * \
                (entry.copied / float(entry.total))
            progress += partialprogress / len(self.entries)

        job.setprogress(progress)

    def run(self):
        source_mount_path = tempfile.mkdtemp()
        try:
            for entry in self.entries:
                imgbasename = os.path.splitext(
                    os.path.basename(entry.source))[0]
                imgmountdir = os.path.join(source_mount_path, imgbasename)
                os.mkdir(imgmountdir)

                self.mount_image(entry, imgmountdir)

                fslist = ""

                if entry.sourcefs == "squashfs":
                    if shutil.which("unsquashfs") == None:
                        return ("Failed to unpack image", "Failed to find unsquashfs, make sure you have "
                        "the squashfs-tools package installed")

                    fslist = subprocess.check_output(["unsquashfs",
                                                        "-l",
                                                        entry.source])
                if entry.sourcefs == "ext4":
                    fslist = subprocess.check_output(["find",
                                                      imgmountdir,
                                                      "-type", "f"])
                entry.total = len(fslist.splitlines())

                self.report_progress()
                error_msg = self.unpack_image(entry, imgmountdir)
                if error_msg:
                    return ("Failed to unpack image {}".format(entry.source),
                        error_msg)
            return None
        finally:
            shutil.rmtree(source_mount_path)

    def mount_image(self, entry, imgmountdir):
        subprocess.check_call(["mount",
                               entry.source,
                               imgmountdir,
                               "-t",
                               entry.sourcefs,
                               "-o", "loop"])

    def unpack_image(self, entry, imgmountdir):
        def progress_cb(copied):
            entry.copied = copied
            self.report_progress()

        try:
            return file_copy(imgmountdir,
                      entry.destination,
                      progress_cb)
        finally:
            subprocess.check_call(["umount", "-l", imgmountdir])


def run():
    # from globalstorage: rootMountPoint
    # from job.configuration:
    # the path to where to mount the source image(s) for copying
    # an ordered list of unpack mappings for image file <-> target dir relative
    # to rootMountPoint, e.g.:
    # configuration:
    #     unpack:
    #         - source: "/path/to/filesystem.img"
    #           sourcefs: "ext4"
    #           destination: ""
    #         - source: "/path/to/another/filesystem.sqfs"
    #           sourcefs: "squashfs"
    #           destination: ""

    PATH_PROCFS = '/proc/filesystems'

    root_mount_point = globalstorage.value("rootMountPoint")
    if not root_mount_point:
        return ("No mount point for root partition in globalstorage",
                "globalstorage does not contain a \"rootMountPoint\" key, "
                "doing nothing")
    if not os.path.exists(root_mount_point):
        return ("Bad mount point for root partition in globalstorage",
                "globalstorage[\"rootMountPoint\"] is \"{}\", which does not "
                "exist, doing nothing".format(root_mount_point))
    unpack = list()

    for entry in job.configuration["unpack"]:
        source = os.path.abspath(entry["source"])

        sourcefs = entry["sourcefs"]

        # Get supported filesystems
        fs_is_supported = False

        if os.path.isfile(PATH_PROCFS) and os.access(PATH_PROCFS, os.R_OK):
            procfile = open(PATH_PROCFS, 'r')
            filesystems = procfile.read()
            procfile.close

            filesystems = filesystems.replace("nodev", "")
            filesystems = filesystems.replace("\t", "")
            filesystems = filesystems.splitlines()

            # Check if the source filesystem is supported
            for fs in filesystems:
                if fs == sourcefs:
                    fs_is_supported = True

        if fs_is_supported == False:
            return "Bad filesystem", "sourcefs=\"{}\"".format(sourcefs)

        destination = os.path.abspath(root_mount_point + entry["destination"])

        if not os.path.exists(source) or os.path.isdir(source):
            return ("Bad source", "source=\"{}\"".format(source))
        if not os.path.isdir(destination):
            return ("Bad destination",
                    "destination=\"{}\"".format(destination))

        unpack.append(UnpackEntry(source, sourcefs, destination))

    unpackop = UnpackOperation(unpack)
    return unpackop.run()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# === This file is part of Calamares - <http://github.com/calamares> ===
#
#   Copyright 2014-2026, Anke Boersma <demm@kaosx.us>
#
#   Calamares is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   Calamares is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with Calamares. If not, see <http://www.gnu.org/licenses/>.

import os
import shutil
import glob
import subprocess
import re

import libcalamares


def run():
    """ Setup graphics drivers and sound """

    install_path = libcalamares.globalstorage.value("rootMountPoint")
    license = libcalamares.globalstorage.value("packagechooser_licenseq")
    print(license)

    if not license or license == 'disagree':
        print('License declined')
        return None

    # remove any db.lck
    db_lock = os.path.join(install_path, "var/lib/pacman/db.lck")
    if os.path.exists(db_lock):
        with misc.raised_privileges():
            os.remove(db_lock)

    # setup proprietary drivers, if detected
    print('setup proprietary drivers')
    if os.path.exists('/var/log/nvidia'):
        print('nvidia detected')
        print('removing unneeded packages')
        libcalamares.utils.target_env_call(
            ['pacman', '-Rdd', '--noconfirm', 'xf86-video-nouveau'])
        print('installing driver')
        shutil.copytree(
            '/opt/kdeos/pkgs', '%s/opt/kdeos/pkgs' % (install_path))
        for nvidia_utils in glob.glob('/opt/kdeos/pkgs/nvidia-utils-2:59*'):
            libcalamares.utils.target_env_call(
                ['pacman', '-Ud', '--noconfirm', nvidia_utils])
        for nvidia in glob.glob('/opt/kdeos/pkgs/nvidia-2:59*'):
            libcalamares.utils.target_env_call(
                ['pacman', '-Ud', '--noconfirm', nvidia])
        shutil.rmtree('%s/opt/kdeos/pkgs' % (install_path))

        sddm_conf_path = os.path.join(install_path, "etc/sddm.conf")
        text = []
        with open(sddm_conf_path, 'r') as sddm_conf:
            text = sddm_conf.readlines()
        with open(sddm_conf_path, 'w') as sddm_conf:
            for line in text:
                if re.match('Session=plasma', line):
                    line = 'Session=plasmax11'
                elif re.match('DisplayServer=wayland', line):
                    line = 'DisplayServer=x11'
                sddm_conf.write(line)

    elif os.path.exists('/var/log/nvidia-prime'):
        print('nvidia prime detected')
        print('removing unneeded packages')
        libcalamares.utils.target_env_call(
            ['pacman', '-Rdd', '--noconfirm', 'xf86-video-nouveau'])
        print('installing driver')
        shutil.copytree(
            '/opt/kdeos/pkgs', '%s/opt/kdeos/pkgs' % (install_path))
        for nvidia_utils in glob.glob('/opt/kdeos/pkgs/nvidia-utils-2:59*'):
            libcalamares.utils.target_env_call(
                ['pacman', '-Ud', '--noconfirm', nvidia_utils])
        for nvidia in glob.glob('/opt/kdeos/pkgs/nvidia-2:59*'):
            libcalamares.utils.target_env_call(
                ['pacman', '-Ud', '--noconfirm', nvidia])
        for prime in glob.glob('/opt/kdeos/pkgs/prime*'):
            libcalamares.utils.target_env_call(
                ['pacman', '-Ud', '--noconfirm', prime])
        shutil.rmtree('%s/opt/kdeos/pkgs' % (install_path))

        sddm_conf_path = os.path.join(install_path, "etc/sddm.conf")
        text = []
        with open(sddm_conf_path, 'r') as sddm_conf:
            text = sddm_conf.readlines()
        with open(sddm_conf_path, 'w') as sddm_conf:
            for line in text:
                if re.match('Session=plasma', line):
                    line = 'Session=plasmax11'
                elif re.match('DisplayServer=wayland', line):
                    line = 'DisplayServer=x11'
                sddm_conf.write(line)

    elif os.path.exists('/var/log/nvidia-580xx'):
        print('nvidia-580xx detected')
        print('removing unneeded packages')
        libcalamares.utils.target_env_call(
            ['pacman', '-Rdd', '--noconfirm', 'xf86-video-nouveau'])
        print('installing driver')
        shutil.copytree(
            '/opt/kdeos/pkgs', '%s/opt/kdeos/pkgs' % (install_path))
        for nvidia_580_utils in glob.glob('/opt/kdeos/pkgs/nvidia-580xx-utils*'):
            libcalamares.utils.target_env_call(
                ['pacman', '-Ud', '--noconfirm', nvidia_580_utils])
        for nvidia_580 in glob.glob('/opt/kdeos/pkgs/nvidia-580xx-58*'):
            libcalamares.utils.target_env_call(
                ['pacman', '-Ud', '--noconfirm', nvidia_580])
        shutil.rmtree('%s/opt/kdeos/pkgs' % (install_path))

        sddm_conf_path = os.path.join(install_path, "etc/sddm.conf")
        text = []
        with open(sddm_conf_path, 'r') as sddm_conf:
            text = sddm_conf.readlines()
        with open(sddm_conf_path, 'w') as sddm_conf:
            for line in text:
                if re.match('Session=plasma', line):
                    line = 'Session=plasmax11'
                elif re.match('DisplayServer=wayland', line):
                    line = 'DisplayServer=x11'
                sddm_conf.write(line)

    elif os.path.exists('/var/log/nvidia-470xx'):
        print('nvidia-470xx detected')
        print('removing unneeded packages')
        libcalamares.utils.target_env_call(
            ['pacman', '-Rdd', '--noconfirm', 'xf86-video-nouveau'])
        print('installing driver')
        shutil.copytree(
            '/opt/kdeos/pkgs', '%s/opt/kdeos/pkgs' % (install_path))
        for nvidia_470_utils in glob.glob('/opt/kdeos/pkgs/nvidia-470xx-utils*'):
            libcalamares.utils.target_env_call(
                ['pacman', '-Ud', '--noconfirm', nvidia_470_utils])
        for nvidia_470 in glob.glob('/opt/kdeos/pkgs/nvidia-470xx-47*'):
            libcalamares.utils.target_env_call(
                ['pacman', '-Ud', '--noconfirm', nvidia_470])
        shutil.rmtree('%s/opt/kdeos/pkgs' % (install_path))

        sddm_conf_path = os.path.join(install_path, "etc/sddm.conf")
        text = []
        with open(sddm_conf_path, 'r') as sddm_conf:
            text = sddm_conf.readlines()
        with open(sddm_conf_path, 'w') as sddm_conf:
            for line in text:
                if re.match('Session=plasma', line):
                    line = 'Session=plasmax11'
                elif re.match('DisplayServer=wayland', line):
                    line = 'DisplayServer=x11'
                sddm_conf.write(line)

    print('done setting up hardware')

    return None
